"use strict";
var TsConfigPathsPlugin = (function () {
    function TsConfigPathsPlugin(source, target) {
        this.source = source;
        this.target = target;
    }
    TsConfigPathsPlugin.prototype.apply = function (resolver) {
        var target = this.target;
        var name = this.name;
        var alias = this.alias;
        var onlyModule = this.onlyModule;
        resolver.plugin(this.source, function (request, callback) {
            var innerRequest = getInnerRequest(resolver, request);
            if (!innerRequest) {
                return callback();
            }
            if ((!onlyModule && innerRequest.indexOf(name + "/") === 0) || innerRequest === name) {
                if (innerRequest.indexOf(alias + "/") !== 0 && innerRequest != alias) {
                    var newRequestStr = alias + innerRequest.substr(name.length);
                    var obj = Object.assign({}, request, {
                        request: newRequestStr
                    });
                    return resolver.doResolve(target, obj, "aliased with mapping '" + name + "': '" + alias + "' to '" + newRequestStr + "'", createInnerCallback(function (err, result) {
                        if (arguments.length > 0) {
                            return callback(err, result);
                        }
                        callback(null, null);
                    }, callback));
                }
            }
            return callback();
        });
    };
    return TsConfigPathsPlugin;
}());
exports.TsConfigPathsPlugin = TsConfigPathsPlugin;
function createInnerCallback(callback, options, message, messageOptional) {
    var log = options.log;
    if (!log) {
        if (options.stack !== callback.stack) {
            var callbackWrapper = function callbackWrapper() {
                return callback.apply(this, arguments);
            };
            callbackWrapper.stack = options.stack;
            callbackWrapper.missing = options.missing;
            return callbackWrapper;
        }
        return callback;
    }
    var loggingCallbackWrapper = function loggingCallbackWrapper() {
        var i;
        if (message) {
            if (!messageOptional || theLog.length > 0) {
                log(message);
                for (i = 0; i < theLog.length; i++) {
                    log("  " + theLog[i]);
                }
            }
        }
        else {
            for (i = 0; i < theLog.length; i++) {
                log(theLog[i]);
            }
        }
        return callback.apply(this, arguments);
    };
    var theLog = [];
    loggingCallbackWrapper.log = function writeLog(msg) {
        theLog.push(msg);
    };
    loggingCallbackWrapper.stack = options.stack;
    loggingCallbackWrapper.missing = options.missing;
    return loggingCallbackWrapper;
}
function getInnerRequest(resolver, request) {
    var innerRequest;
    if (request.request) {
        innerRequest = request.request;
        if (/^\.\.?\//.test(innerRequest) && request.relativePath) {
            innerRequest = resolver.join(request.relativePath, innerRequest);
        }
    }
    else {
        innerRequest = request.relativePath;
    }
    return innerRequest;
}
;
//# sourceMappingURL=alias-resolver.js.map